import { useEffect, useMemo, useRef, useState } from "react";
import { EventArgs, Graph, Shape } from "@antv/x6";
import { History } from "@antv/x6-plugin-history";
import { Transform } from "@antv/x6-plugin-transform";
import { Scroller } from "@antv/x6-plugin-scroller";
import { Snapline } from "@antv/x6-plugin-snapline";
import { GetAllDesignTables } from "@/api";
import { useRequest } from "umi";
import type {
  ColumnItem,
  ColumnRelation,
  ProjectInfo,
  RemarkInfo,
  TableItemType,
  TopicAreaInfo,
} from "@/type";
import { uuid } from "@/utils";
import { RelationType } from "@/enum";

import "@/components/TableNode";
import "@/components/TopicNode";
import "@/components/NoticeNode";
import { message } from "antd";

export default function erModel() {
  const graphRef = useRef<Graph>();
  const [graph, setGraph] = useState<Graph>();
  const [project, setProject] = useState<ProjectInfo>({
    id: "1",
    name: "项目1",
    folderId: "root",
    creationTime: "2024-12-12 12:12:12",
    creatorUser: "管理员",
    lastModificationTime: "2024-12-12 12:12:12",
    description: "模型描述",
    isTemplate: false,
    industry: "互联网",
    publishStatus: "未发布",
    tables: [],
    relations: [],
    topicAreas: [],
    remarks: [],
    history: [],
    setting: {
      showMenu: true,
      showSidebar: true,
      showColumnDetail: true,
      showGrid: true,
      showRelation: true,
    },
  });

  const initGraph = (container: HTMLElement) => {
    const instance = new Graph({
      container,
      width: document.documentElement.clientWidth,
      height: document.documentElement.clientHeight,
      autoResize: true,
      async: false,
      mousewheel: {
        enabled: true,
        modifiers: "ctrl",
        minScale: 0.2,
        maxScale: 2,
      },
      highlighting: {
        nodeAvailable: {
          name: "stroke",
          args: {
            padding: 4,
            attrs: {
              "stroke-width": 2,
              stroke: "red",
            },
          },
        },
      },
      connecting: {
        allowBlank: false,
        allowEdge: false,
        allowLoop: false,
        router: {
          name: "normal",
        },
        createEdge() {
          return new Shape.Edge({
            attrs: {
              line: {
                stroke: "#ff0000",
                strokeWidth: 1,
                strokeDasharray: 5,
                targetMarker: null,
              },
            },
            data: {
              type: "refer",
            },
          });
        },
      },
      grid: {
        visible: true,
        size: 10,
      },
      background: {
        color: "#F2F7FA",
      },
      interacting: {
        nodeMovable: (view) => {
          const data = view.cell.getData<{
            ignoreDrag: boolean;
            lock: boolean;
          }>();
          // 禁止拖拽或锁节点
          if (data?.ignoreDrag || data?.lock) return false;
          return true;
        },
      },
    });

    instance.use(new History());
    instance.use(new Snapline({ enabled: true }));
    instance.use(
      new Transform({
        resizing: {
          enabled: true,
        },
      })
    );
    instance.use(new Scroller());

    setGraph(instance);
    graphRef.current = instance;

    instance.on(
      "node:change:add:relation",
      (args: EventArgs["cell:change:*"]) => {
        console.log("node:change:add:relation", args.current);
        const { source, target } = args.current;
        if (source && target) {
          addRelation(source, target);
        }
      }
    );
  };

  /**
   * 添加表
   */
  const addTable = () => {
    const tableId = uuid();
    const columnId = uuid();
    const newTable: TableItemType = {
      table: {
        aliasName: "newtable",
        creationTime: "",
        creatorUserId: "",
        displayOrder: 1,
        id: tableId,
        isDeleted: false,
        langDescription: "",
        langName: "",
        name: "",
        cn_name: "新建表",
        en_name: "new table",
        parentBusinessTableId: "",
        schemaName: "new_table",
        type: 1,
        updateTime: "",
        openSync: false,
        style: {
          color: "#616161",
          x: 0,
          y: 0,
        },
      },
      tableColumnList: [
        {
          aggregateEnable: false,
          businessTableId: tableId,
          charset: "",
          defaultValue: "",
          displayEnable: false,
          displayOrder: 1,
          groupByEnable: false,
          id: columnId,
          isAggregateEnable: false,
          isDisplayEnable: false,
          isGroupByEnable: false,
          isLinkEnable: false,
          isOrderByEnable: false,
          isPreDefined: true,
          isRequired: true,
          isUnique: false,
          isWhereEnable: false,
          langName: "120d5c4e-a5d8-4d49-b5f5-492feb37e6c6",
          linkEnable: false,
          maxLength: 50,
          name: "Id",
          orderByEnable: false,
          preDefined: true,
          precision: 0,
          required: true,
          scale: 0,
          schemaName: "id",
          type: 2,
          unique: false,
          whereEnable: false,
          cn_name: "id",
          en_name: "id",
        },
      ],
    };

    setProject({
      ...project,
      tables: [...project.tables, newTable],
    });

    graphRef.current?.addNode({
      shape: "table-node",
      x: 300,
      y: 100,
      width: 200,
      height: 200,
      id: tableId,
      data: newTable,
      zIndex: 3,
      ports: {
        groups: {
          // 字段名前连接桩
          columnPort: {
            markup: [
              {
                tagName: "rect",
                selector: "rect",
              },
              {
                tagName: "circle",
                selector: "circle",
              },
            ],
            position: {
              name: "absolute",
              args: {
                x: 12,
                y: 42,
              },
            },
          },
        },
      },
    });
  };

  /**
   * 更新表
   * @param table
   */
  const updateTable = (table: TableItemType) => {
    setProject({
      ...project,
      tables: project.tables.map((item) => {
        if (item.table.id === table.table.id) {
          const tableNode = graphRef.current?.getCellById(table.table.id);
          tableNode?.setData(table);
          return table;
        }
        return item;
      }),
    });
  };

  /**
   * 删除表
   * @param tableId
   */
  const deleteTable = (tableId: string) => {
    setProject({
      ...project,
      tables: project.tables.filter((item) => item.table.id !== tableId),
      relations: project.relations.filter(
        (item) => item.primaryTable !== tableId && item.foreignTable !== tableId
      ),
    });
    graphRef.current?.removeCell(tableId);
    // 删除关联关系
    project.relations.forEach((item) => {
      if (item.primaryTable === tableId || item.foreignTable === tableId) {
        graphRef.current?.removeCell(item.id);
      }
    });
  };

  /**
   * 增加主题域
   */
  const addTopicArea = () => {
    const topicAreaId = uuid();
    const newTopicArea = {
      id: topicAreaId,
      name: "主题域_" + (project.topicAreas.length + 1),
      style: {
        background: "#175e7a",
      },
    };
    setProject({
      ...project,
      topicAreas: [...project.topicAreas, newTopicArea],
    });

    graphRef.current?.addNode({
      shape: "topic-node",
      x: 300,
      y: 100,
      width: 200,
      height: 200,
      id: topicAreaId,
      data: newTopicArea,
      zIndex: 0,
    });
  };

  /**
   * 修改主题域
   */
  const updateTopicArea = (topicArea: TopicAreaInfo) => {
    setProject({
      ...project,
      topicAreas: project.topicAreas.map((item) => {
        if (item.id === topicArea.id) {
          const topicAreaNode = graphRef.current?.getCellById(topicArea.id);
          topicAreaNode?.setData(topicArea);
          return topicArea;
        }
        return item;
      }),
    });
  };

  /**
   * 删除主题域
   * @param topicAreaId
   */
  const deleteTopicArea = (topicAreaId: string) => {
    setProject({
      ...project,
      topicAreas: project.topicAreas.filter((item) => item.id !== topicAreaId),
    });
    graphRef.current?.removeCell(topicAreaId);
  };

  /**
   * 添加备注
   */
  const addRemark = () => {
    const remarkId = uuid();
    const newRemark = {
      id: remarkId,
      name: "备注_" + (project.remarks.length + 1),
      text: "",
      style: {
        x: 0,
        y: 0,
        width: 200,
        height: 100,
        background: "#fcf7ac",
      },
    };
    setProject({
      ...project,
      remarks: [...project.remarks, newRemark],
    });

    const notice = graphRef.current?.addNode({
      shape: "notice-node",
      x: 300,
      y: 100,
      width: 200,
      height: 200,
      id: remarkId,
      data: newRemark,
      zIndex: 1,
    });

    notice?.on("change:data", function (args) {
      updateRemark(args.current);
    });
  };

  /**
   * 修改备注
   */
  const updateRemark = (remark: RemarkInfo) => {
    setProject((state) => ({
      ...(state || {}),
      remarks: state.remarks.map((item) => {
        if (item.id === remark.id) {
          const remarkNode = graphRef.current?.getCellById(remark.id);
          remarkNode?.setData(remark);
          return remark;
        }
        return item;
      }),
    }));
  };

  /**
   * 删除备注
   * @param remarkId
   */
  const deleteRemark = (remarkId: string) => {
    setProject({
      ...project,
      remarks: project.remarks.filter((item) => item.id !== remarkId),
    });
    graphRef.current?.removeCell(remarkId);
  };

  const getRelations = (project: ProjectInfo, newRelation: ColumnRelation) => {
    let sourceColumn: ColumnItem | undefined;
    let targetColumn: ColumnItem | undefined;
    let sourceTable: TableItemType | undefined;
    let targetTable: TableItemType | undefined;
    project.tables.forEach((table) => {
      if (table.table.id === newRelation.primaryTable) {
        sourceTable = table;
        sourceColumn = table.tableColumnList.find(
          (item) => item.id === newRelation.primaryKey
        );
      }
      if (table.table.id === newRelation.foreignTable) {
        targetTable = table;
        targetColumn = table.tableColumnList.find(
          (item) => item.id === newRelation.foreignKey
        );
      }
    });
    console.log(sourceColumn, targetColumn, newRelation);
    if (!sourceColumn || !targetColumn) {
      return {
        relations: project.relations,
        canAdd: false,
      };
    }
    if (sourceColumn.type !== targetColumn.type) {
      message.warning("数据类型不一致");
      return {
        relations: project.relations,
        canAdd: false,
      };
    }

    return {
      relations: [
        ...project.relations, {
          ...newRelation,
          name: `${sourceTable?.table.schemaName}_${targetTable?.table.schemaName}_${sourceColumn.schemaName}`,
        }],
      canAdd: true,
    };
  };

  const addRelationEdge = (
    id: string,
    source: RelationItem,
    target: RelationItem
  ) => {
    // 添加关系连线
    const relationEdge = graphRef.current?.addEdge({
      id,
      router: {
        name: "manhattan",
        args: {
          direction: "H",
        },
      },
      attrs: {
        line: {
          stroke: "#333",
          strokeWidth: 1,
          targetMarker: null,
        },
      },
      source: {
        cell: source.tableId,
        port: source.columnId + "_port2",
        anchor: "left",
      },
      target: {
        cell: target.tableId,
        port: target.columnId + "_port2",
        anchor: "left",
      },
      data: {
        type: "relation",
        label: uuid(),
      },
      defaultLabel: {
        markup: [
          {
            tagName: "circle",
            selector: "bg",
          },
          {
            tagName: "text",
            selector: "txt",
          },
        ],
        attrs: {
          txt: {
            fill: "#fff",
            textAnchor: "middle",
            textVerticalAnchor: "middle",
          },
          bg: {
            ref: "txt",
            fill: "#333",
            r: 10,
            strokeWidth: 0,
          },
        },
      },
    });
    relationEdge?.appendLabel({
      attrs: {
        txt: {
          text: 1,
        },
      },
      position: {
        distance: 25,
      },
    });
    relationEdge?.appendLabel({
      attrs: {
        txt: {
          text: 1,
        },
      },
      position: {
        distance: -25,
      },
    });
  };

  type RelationItem = {
    tableId: string;
    columnId: string;
  };

  /**
   * 添加关系
   */
  const addRelation = (source: RelationItem, target: RelationItem) => {
    const newRelation: ColumnRelation = {
      id: uuid(),
      name: "",
      primaryKey: source.columnId,
      primaryTable: source.tableId,
      foreignKey: target.columnId,
      foreignTable: target.tableId,
      relationType: 1,
      style: {},
    };
    setProject((state) => {
      const obj = getRelations(state, newRelation);
      if (obj.canAdd) {
        // 添加连线
        addRelationEdge(newRelation.id, source, target);

        return {
          ...state,
          relations: obj.relations,
        };
      } else {
        return state;
      }
    });
  };

  /**
   * 更新关系
   */
  const updateRelation = (relation: ColumnRelation) => {
    setProject((state) => {
      return {
        ...state,
        relations: state.relations.map((item) => {
          if (item.id === relation.id) {
            return relation;
          }
          return item;
        }),
      };
    });
    // 更新连线
    const relationEdge = graphRef.current?.getCellById(relation.id);
    if (relationEdge?.isEdge()) {
      relationEdge.setSource({
        cell: relation.primaryTable,
        port: relation.primaryKey + "_port2",
        anchor: "left",
      });
      relationEdge.setTarget({
        cell: relation.foreignTable,
        port: relation.foreignKey + "_port2",
        anchor: "left",
      });
      // 更新标签
      relationEdge.setLabelAt(0, {
        attrs: {
          txt: {
            text: relation.relationType === RelationType.OneToOne || relation.relationType === RelationType.OneToMany ? '1' : 'n',
          },
        },
        position: {
          distance: 25,
        },
      });
      relationEdge.setLabelAt(1, {
        attrs: {
          txt: {
            text: relation.relationType === RelationType.OneToMany || relation.relationType === RelationType.ManyToMany ? 'n' : '1',
          },
        },
        position: {
          distance: -25,
        },
      });
    }
  };

  /**
   * 删除关系
   */
  const deleteRelation = (relationId: string) => {
    setProject({
      ...project,
      relations: project.relations.filter((item) => item.id !== relationId),
    });
    // 删除连线
    graphRef.current?.removeCell(relationId);
  };

  return {
    initGraph,
    graph,
    graphRef,
    project,
    setProject,
    addTable,
    updateTable,
    deleteTable,
    addTopicArea,
    updateTopicArea,
    deleteTopicArea,
    addRemark,
    updateRemark,
    deleteRemark,
    addRelation,
    updateRelation,
    deleteRelation,
  };
}
