import {
  DeleteOutlined,
  HolderOutlined,
} from "@ant-design/icons";
import {
  Col,
  Row,
  Form,
  Input,
  InputNumber,
  Select,
  Button,
  Tooltip,
  Popover,
  Popconfirm,
} from "antd";
import React, { useState } from "react";
import CustomColorPicker from "@/components/CustomColorPicker";
import { ColumnItem, TableItemType } from "@/type";
import { TABLE_TYPE_OPTIONS, DATA_TYPE_OPTIONS } from "@/constants";
import { uuid } from "@/utils";
import { DataType } from "@/enum";
import { useModel } from "umi";

export default function TableItem({
  data,
  onChange,
}: {
  data: TableItemType;
  onChange: (data: TableItemType) => void;
}) {
  const [collapsed, setCollapsed] = React.useState(false);
  const { tableColumnList = [] } = data;
  const { addTable, deleteTable } = useModel("erModel");

  const [table, setTable] = React.useState(data?.table);

  const handleTableChange = (key: string, value: any) => {
    onChange({
      tableColumnList,
      table: { ...table, [key]: value },
    });
    setTable({ ...table, [key]: value });
  };

  const handleChangeColumn = (index: number, key: string, value: any) => {
    onChange({
      table,
      tableColumnList: tableColumnList.map((item, i) => {
        if (index === i) {
          return { ...item, [key]: value };
        }
        return item;
      }),
    })
  };

  // 添加表
  const handleAddChildTable = () => {
    addTable();
  };

  // 添加字段
  const handleAddColumn = () => {
    const newColumn: ColumnItem = {
      id: uuid(),
      schemaName: "",
      name: "",
      en_name: "",
      cn_name: "",
      type: DataType.Nvarchar,
      maxLength: 100,
      precision: 0,
      scale: 0,
      isRequired: false,
      isUnique: false,
      isPreDefined: false,
      defaultValue: "",
    };
    onChange({
      table,
      tableColumnList: [...tableColumnList, newColumn],
    });
  };

  return (
    <div
      className="
      w-full 
      border-l-solid 
      border-l-[#e6e6e6] 
      border-l-[4px] 
      border-b-solid 
      border-b-[#e4e4e4] 
      border-b-[1px] 
      m-b-4px
      p-l-16px"
      style={{
        borderLeftColor: table.style?.color || "#eee",
      }}
    >
      <div
        className="
        header 
        flex 
        items-center 
        justify-between 
        leading-[40px] 
        px-[10px] 
        hover:bg-[#fafafa] 
        cursor-pointer 
        m-b-[10px]"
        onClick={() => setCollapsed(!collapsed)}
      >
        <div className="font-bold">
          {table.schemaName}({table.cn_name})
        </div>
        <div>
          <Popover
            trigger="click"
            placement="right"
            content={
              <div className="w-200px" onClick={(e) => e.stopPropagation()}>
                <Form layout="vertical">
                  <Form.Item label="表名称" name="pkName">
                    <Input
                      className="w-full"
                      placeholder="中文"
                      value={table.cn_name}
                      onChange={(e) =>
                        handleTableChange("cn_name", e.target.value)
                      }
                    />
                    <Input
                      className="w-full"
                      placeholder="英文"
                      value={table.en_name}
                      onChange={(e) =>
                        handleTableChange("en_name", e.target.value)
                      }
                    />
                  </Form.Item>
                  <Form.Item label="描述" name="pkName">
                    <Input.TextArea className="w-full" placeholder="中文..." />
                    <Input.TextArea className="w-full" placeholder="英文..." />
                  </Form.Item>
                </Form>
              </div>
            }
          >
            <i
              className="iconfont icon-shezhi mr-[10px] cursor-pointer"
              onClick={(e) => e.stopPropagation()}
            />
          </Popover>
          <i className="iconfont icon-open" />
        </div>
      </div>
      <div
        className="content overflow-hidden"
        style={{ height: collapsed ? 0 : "auto" }}
      >
        <Form layout="horizontal" labelCol={{ span: 8 }}>
          <Row gutter={8}>
            <Col span={12}>
              <Form.Item label="编码">
                <Input
                  placeholder="请输入"
                  value={table.schemaName}
                  onChange={(e) =>
                    handleTableChange("schemaName", e.target.value)
                  }
                />
              </Form.Item>
            </Col>
            <Col span={12}>
              <Form.Item label="别名">
                <Input
                  placeholder="请输入"
                  value={table.aliasName}
                  onChange={(e) =>
                    handleTableChange("aliasName", e.target.value)
                  }
                />
              </Form.Item>
            </Col>
          </Row>
          <Form.Item
            label="类型"
            labelCol={{ span: 4 }}
            wrapperCol={{ span: 21 }}
          >
            <Select
              placeholder="请选择"
              options={TABLE_TYPE_OPTIONS}
              value={table.type}
              onChange={(val) => handleTableChange("type", val)}
            />
          </Form.Item>
        </Form>

        <div className="flex justify-between m-b-10px">
          <CustomColorPicker
            color={table.style?.background}
            onChange={(color) =>
              handleTableChange("style", { ...table.style, color })
            }
          >
            <div
              className="rounded-4px cus-btn w-32px h-32px bg-#eee flex-none cursor-pointer shadow-inner"
              style={{ background: table.style?.color || "#eee" }}
            ></div>
          </CustomColorPicker>
          <div className="flex gap-4px">
            <Button type="primary" onClick={handleAddChildTable}>
              添加子表
            </Button>
            <Button type="primary" onClick={handleAddColumn}>
              添加字段
            </Button>
            <Popconfirm
              okType="primary"
              title="确定删除该表？"
              okText="确定"
              cancelText="取消"
              onConfirm={() => deleteTable(table.id)}
            >
              <div className="rounded-4px cus-btn w-32px h-32px bg-#eee flex-none text-center leading-32px color-red cursor-pointer">
                <DeleteOutlined />
              </div>
            </Popconfirm>
          </div>
        </div>

        <div className="column-content border-solid border-1px border-#e4e4e4 border-x-none p-y-10px">
          {/* 字段内容 */}
          {tableColumnList.map((column, index) => {
            return (
              <div key={column.id} className="column-item flex gap-4px items-center jutify-space-between hover:bg-gray-100 mb-4px">
                <HolderOutlined className="cursor-move" />
                <Tooltip title="字段编码">
                  <Input
                    placeholder="编码"
                    value={column.schemaName}
                    className="flex-1"
                    onChange={(e) => handleChangeColumn(index, "schemaName", e.target.value)}
                  />
                </Tooltip>
                <Tooltip title="字段类型">
                  <Select
                    placeholder="类型"
                    className="w-80px"
                    options={DATA_TYPE_OPTIONS}
                    value={column.type}
                    onChange={(value) => handleChangeColumn(index, "type", value)}
                    dropdownStyle={{ width: 120 }}
                  />
                </Tooltip>
                <Tooltip title="非空">
                  <div
                    className="
                    rounded-4px 
                    cus-btn 
                    w-32px 
                    h-32px 
                    bg-#eee 
                    flex-none 
                    text-center 
                    leading-32px 
                    cursor-pointer 
                    hover:bg-#ddd"
                    style={
                      column.isRequired
                        ? { background: "#1677ff", color: "#fff" }
                        : {}
                    }
                    onClick={() => handleChangeColumn(index, "isRequired", !column.isRequired)}
                  >
                    !
                  </div>
                </Tooltip>
                <Tooltip title="唯一">
                  <div
                    className="
                    rounded-4px 
                    cus-btn 
                    w-32px 
                    h-32px 
                    bg-#eee 
                    flex-none 
                    text-center 
                    leading-32px 
                    cursor-pointer 
                    hover:bg-#ddd"
                    style={
                      column.isUnique
                        ? { background: "#1677ff", color: "#fff" }
                        : {}
                    }
                    onClick={() => handleChangeColumn(index, "isUnique", !column.isUnique)}
                  >
                    1
                  </div>
                </Tooltip>
                <Popover
                  trigger="click"
                  placement="right"
                  content={
                    <div
                      className="w-200px"
                      onClick={(e) => e.stopPropagation()}
                    >
                      <Form layout="vertical">
                        <Form.Item label="字段名称" name="pkName">
                          <Input
                            className="w-full"
                            placeholder="中文"
                            value={column.cn_name}
                            onChange={(e) =>
                              handleChangeColumn(index, "cn_name", e.target.value)
                            }
                          />
                          <Input
                            className="w-full"
                            placeholder="英文"
                            value={column.en_name}
                            onChange={(e) =>
                              handleChangeColumn(index, "en_name", e.target.value)
                            }
                          />
                        </Form.Item>
                        <Form.Item label="描述" name="pkName">
                          <Input.TextArea
                            className="w-full"
                            placeholder="描述中文..."
                          />
                          <Input.TextArea
                            className="w-full"
                            placeholder="描述英文..."
                          />
                        </Form.Item>
                        <Form.Item label="长度">
                          <InputNumber
                            placeholder="请输入"
                            min={0}
                            className="w-full"
                            value={column.maxLength}
                            onChange={(num) =>
                              handleChangeColumn(index, "maxLength", num)
                            }
                          />
                        </Form.Item>
                        <Form.Item label="精度">
                          <InputNumber
                            placeholder="请输入"
                            min={0}
                            className="w-full"
                            value={column.precision}
                            onChange={(num) =>
                              handleChangeColumn(index, "precision", num)
                            }
                          />
                        </Form.Item>
                        <Form.Item label="默认值" name="pkName">
                          <Input
                            className="w-full"
                            placeholder="默认值"
                            value={column.defaultValue}
                            onChange={(e) =>
                              handleChangeColumn(index, "defaultValue", e.target.value)
                            }
                          />
                        </Form.Item>
                      </Form>
                    </div>
                  }
                >
                  <div className="rounded-4px cus-btn w-32px h-32px bg-#eee flex-none text-center leading-32px cursor-pointer hover:bg-#ddd">
                    <i className="iconfont icon-gengduo" />
                  </div>
                </Popover>
              </div>
            );
          })}
        </div>
      </div>
    </div>
  );
}
